/********************************************************************************/
/*                        MIDI SHARE Application Skeleton                       */
/*							Version Think C 4.0									*/
/*------------------------------------------------------------------------------*/
/* MIDISHARE est un module logiciel conu spcialement pour permettre le fonc-	*/
/* tionnement et le dveloppement d'applications Midi dans un contexte multi-	*/
/* tches et temps rel.	 													*/
/* 																				*/
/* MIDISHARE prend en charge la plupart des problmes spcifiques lis au dve- */
/* loppement d'applications Midi: gestion des flux Midi, gestion du temps,		*/
/* gestion des tches. En outre, MIDISHARE permet le fonctionnement simultan	*/
/* sous Multi-Finder de ces applications, ainsi que l'tablissement de connec-	*/
/* tions Midi virtuelles entre celles-ci.										*/
/* 																				*/
/* SKELETON est un exemple d'application MidiShare minimale qui peut tre		*/
/* utilise comme base de dveloppement pour de nouvelles applications.			*/
/* Par dfaut, l'application renvoie tous les vnements qu'elle reoit, sans	*/
/* aucune modification.															*/
/*																				*/
/* Les points qui peuvent tre facilement modifis sont les suivants :			*/
/*																				*/
/* - La constante ApplName : le nom de l'application pour MidiShare			 	*/
/*																				*/
/* - ReceiveEvents : qui traite tous les vnements reus et qui ralise ici 	*/
/*   un simple MidiThru.														*/
/*																				*/
/* - La constante wantRealTime : quand elle vaut 1, ReceiveEvents est appele	*/
/*   sous interruption et fonctionne donc en temps rel, mais ne peut pas		*/
/*   utiliser les appels  la Toolbox; quand elle vaut 0, ReceiveEvents est 	*/
/*   appele dans la boucle principale et peut donc utiliser la toolbox.		*/
/*																				*/
/* - La constante wantFilter : quand elle vaut 1, un filtre sur les entres  	*/
/*   Midi est install, quand elle vaut 0, aucun filtre n'est install et 		*/
/*   l'application reoit tous les vnements.									*/
/*																				*/
/* - InstallFilter : qui configure le filtre d'entre							*/
/*																				*/
/* - La resource de dialogue 128 qui dfinit la fentre de l'application		*/
/*																				*/
/*------------------------------------------------------------------------------*/
/*		 GRAME 1987-92, Yann Orlarey et Herv Lequay 							*/
/********************************************************************************/

#include "MIDIShare.h"

#include <Values.h>
#include <Types.h>
#include <Resources.h>
#include <QuickDraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Windows.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <Desk.h>
#include <ToolUtils.h>
#include <Memory.h>
#include <SegLoad.h>
#include <Files.h>
#include <OSUtils.h>
#include <OSEvents.h>
#include <DiskInit.h>
#include <Packages.h>
#include <Traps.h>

/* Quelques constantes Macintosh */

#define nil						0
#define osEvent 				app4Evt
#define	resumeMask				1
#define	suspendResumeMessage	1
#define mouseMovedMessage		0xFA
#define _Unimplemented			0xA89F			/* Unimplemented trap				*/
#define _WaitNextEvent			0xA860			/* WaitNextEvent trap				*/


/* Une barre de menus minimale */

#define mBarSize	3
#define	AppleMenu	0
#define	FileMenu	1
#define	EditMenu	2

#define	AppleID		128
#define	FileID		129
#define	EditID		130


/* Les items du menu Edit	*/

#define	UndoI		1
#define	CutI		3
#define	CopyI		4
#define	PasteI		5
#define	ClearI		6


/* La fentre de l'application qui est en fait un dialog */

#define WindowID	128 						


/* Pour les erreurs et le About	*/

#define	AboutID		129
#define	AlertID		500


/* Constantes pour paramtrer l'application	*/

#define ApplName 		"\pSkeleton"
#define	wantRealTime 	1
#define wantFilter		0


/* Variables globales pour MidiShare	*/

short		myRefNum;							/* numro d'appl. Midi				*/
TFilter		myFilter;							/* filtre pour les vnements Midi	*/


/* Variables globales d'une application Macintosh	*/

WindowPtr	myWindow;							/* fentre							*/

Boolean		doneFlag;							/* flag d'arrt						*/
	
SysEnvRec	gMac;								/* machine							*/
Boolean		hasWNE;								/* vrai si WaitNextEvent implment	*/
Boolean		foreGround;							/* vrai si en foreGround			*/

EventRecord	myEvent;							/* pour la main event loop			*/
char		theChar;							/* pour la gestion des touches		*/
Rect		dragRect;							/* rect de dplacement de fentre	*/
MenuHandle	myMenus[mBarSize];					/* les menus						*/





/********************************************************************************/
/* 								Macintosh	 utilities							*/
/*------------------------------------------------------------------------------*/
/* 																				*/
/********************************************************************************/

void AlertUser (short r)						/* Signale une erreur  l'utilisateur	*/
{	
	Str255		msg;
	
	GetIndString( msg, AlertID, r);
	ParamText( ApplName, msg, "\p", "\p");
	Alert(AlertID,nil);
	ExitToShell();								
}

Boolean		TrapAvailable(short	trapNum, short tType)
{
	if ( (tType == ToolTrap)  &&						/* Toolbox trap		*/
		 (gMac.machineType > envMachUnknown)  &&		/* 512KE, Plus, or	*/
		 (gMac.machineType < envMacII) ) {				/* SE				*/
		 
		trapNum &= 0x3FF;				/* Traps numbers are 10 bits long	*/
		
		if (trapNum > 0x1FF) {			/* Traps only go up to 0x1FF on		*/
			return(false);				/*   these machines					*/
		}
	}
	return NGetTrapAddress(trapNum, tType) != GetTrapAddress(_Unimplemented);
}

Boolean IsAppWindow (WindowPtr wind)
{
	return  wind ? ((WindowPeek)wind)->windowKind >= 0 : false ;
}

Boolean IsDAWindow (WindowPtr wind)
{
	return  wind ? ((WindowPeek)wind)->windowKind < 0 : false ;
}

void CloseWind (WindowPtr wind)
{
	if ( IsDAWindow(wind) )
		CloseDeskAcc( ((WindowPeek)wind)->windowKind );
	else if ( IsAppWindow(wind) )
		DisposeWindow(wind);
}

void CloseAllWinds (WindowPtr wind)
{
	if (wind) {
		CloseAllWinds( (WindowPtr) (((WindowPeek)wind)->nextWindow) );
		CloseWind(wind);
	}
}



/*=============================== MidiShare Part ===============================*/


/********************************************************************************/
/* 								ReceiveEvents									*/
/*------------------------------------------------------------------------------*/
/* Cette procdure traite tous les vnements reus. Elle peut tre appele		*/
/* "manuellement" (wantRealTime 0) dans la boucle principale de l'application	*/
/* s'il n'y a pas de contraintes temps rel sur la rception.					*/
/* 																				*/
/* Dans le cas contraire (wantRealTime 1), grce  MidiSetRcvAlarm, la routine 	*/
/* peut tre appele automatiquement par MidiShare, sous interruptions, chaque 	*/ 
/* fois que l'application reoit de nouveaux vnements (voir SetupMidi). 		*/
/* 																				*/
/* Dans le code ci dessous, ReceiveEvents ralise un simple MidiThru sans		*/
/* aucun traitement. 															*/
/* 																				*/
/* ReceiveEvents doit obligatoirement comporter un paramtre, qui est le 		*/
/* numro de rfrence MidiShare de l'application.								*/
/* 																				*/
/* Les paramtres de l'appel :													*/
/* ---------------------------													*/
/* 																				*/
/*		refNum:	numro de rfrence MidiShare de l'application.					*/
/* 																				*/
/********************************************************************************/

pascal void ReceiveEvents (register short r)
{
	register MidiEvPtr	e;
	register long n;

	if (n=MidiCountEvs(r))						/* Pour chaque vnement reu :		*/
		do {
			e = MidiGetEv(r);						/*  rcuprer l'vnement			*/
			/* .... */								/*  traiter l'vnement				*/
			MidiSend(r,e);							/*  renvoyer l'vnement			*/
		}while (--n);
}


/****************************************************************************
* 							InstallFilter									*
*---------------------------------------------------------------------------*
* Cette procdure dfinit les valeurs du filtre de l'application. Un filtre *
* est compos de trois parties, qui sont trois tableaux de boolens :		* 
* 																			*
*		un tableau de 256 bits pour les ports Midi accepts					*
*		un tableau de 256 bits pour les types d'vnements accepts			*
*		un tableau de  16 bits pour les canaux Midi accepts				*
* 																			*
* Dans le code ci dessous, le filtre est paramtr pour accepter n'importe	*
* quel type d'vnement. 													*
* 																			*
* Les paramtres de l'appel :												*
* ---------------------------												*
* 																			*
*		aucun																*
* 																			*
*****************************************************************************/

void InstallFilter ()
{
	register short i;

	for (i = 0; i<256; i++)
	{ 										
		AcceptBit(myFilter.evType,i);				/* accepte tous les types d'vnements	*/
		AcceptBit(myFilter.port,i);					/* en provenance de tous les ports		*/
	}
											
	for (i = 0; i<16; i++)
		AcceptBit(myFilter.channel,i);				/* et sur tous les canaux Midi			*/
		
	MidiSetFilter( myRefNum, &myFilter );   		/* installe le filtre					*/
}


/********************************************************************************/
/* 								SETUP MIDI										*/
/*------------------------------------------------------------------------------*/
/* Vrifie la prsence de MidiShare, la possibilit d'ouverture Midi, et		*/
/* installe la procdure d'alarme qui sera appele par MidiShare lors des chan-	*/
/* gements de statut des applications ou des ports.								*/
/* 																				*/
/* Les paramtres de l'appel :													*/
/* ---------------------------													*/
/* 																				*/
/*		aucun																	*/
/* 																				*/
/********************************************************************************/

void SetUpMidi()
{
	if (!MidiShare())								/* Vrifie la prsence de MidiShare			*/
	{
		AlertUser(1);
	}
	
	myRefNum= MidiOpen(ApplName);					/* Ouvre l'application pour MidiShare		*/		
	if (myRefNum == MIDIerrSpace)					/* Vrifie que tout c'est bien pass		*/
	{
		AlertUser(2);
	} 
	
	if (wantRealTime)								/* Installe la routine de rception pour	*/
		MidiSetRcvAlarm(myRefNum, ReceiveEvents);	/* qu'elle soit appele sous interruptions 	*/
		
	if (wantFilter)
		InstallFilter();							/* Installe le filtre de rception			*/
		
	MidiConnect(0,myRefNum,true);					/* Connecte l'entre de l'appl. aux entres physiques	*/
	MidiConnect(myRefNum,0,true);					/* Connecte la sortie de l'appl. aux sorties physiques	*/
}
			

/********************************************************************************/
/* 								Do Idle											*/
/*------------------------------------------------------------------------------*/
/* Execution des diffrentes tches Midi : reception des vnements Midi si		*/
/* cela n'est pas fait sous interruptions; Excution des tches diffres en	*/
/* attente (tches cres par MidiDTask).										*/
/* 																				*/
/* Les paramtres de l'appel :													*/
/* ---------------------------													*/
/* 																				*/
/*		aucun																	*/
/* 																				*/
/********************************************************************************/
void DoIdle()
{
	register long	n;
	register short 	r;
	
	r = myRefNum;
	if (!wantRealTime) ReceiveEvents(r);
	for (n=MidiCountDTasks(r); n>0; --n) MidiExec1DTask(r);
	/* .... */
}


/********************************************************************************/
/* 								Setup windows									*/
/*------------------------------------------------------------------------------*/
/* Ouvre la fentre																*/
/* 																				*/
/* Les paramtres de l'appel :													*/
/* ---------------------------													*/
/* 																				*/
/*		aucun																	*/
/* 																				*/
/********************************************************************************/

void SetUpWindows()
{
	SetRect (&dragRect, qd.screenBits.bounds.left, qd.screenBits.bounds.top, 
						qd.screenBits.bounds.right,qd.screenBits.bounds.bottom);
	InsetRect (&dragRect, 4, 4);

 	myWindow = GetNewDialog (WindowID, nil, (WindowPtr) -1);
 	SetPort(myWindow);
	TextFont(monaco); TextSize(9); TextMode(srcCopy); PenNormal();
	ShowWindow(myWindow);

}

/********************************************************************************
* 								TrackDialog										*
*------------------------------------------------------------------------------	*
* Procdure charge du suivi de la souris.										*
* 																				*
* Les paramtres de l'appel :													*
* ---------------------------													*
* 																				*
*		w	:	pointeur sur la fentre de dialogue.							*
*		p	:	point o la souris a t clique.								*
* 																				*
********************************************************************************/

void TrackDialog(WindowPtr w, Point p)
{
	SetPort(w);
	GlobalToLocal(&p);								/* conversion du point en local	*/
}

		
/********************************************************************************/
/* 								SET UP MENUS									*/
/*------------------------------------------------------------------------------*/
/* Installation de la barre de menus										  	*/
/* 																				*/
/* Les paramtres de l'appel :													*/
/* ---------------------------													*/
/* 																				*/
/*		aucun																	*/
/* 																				*/
/********************************************************************************/

void SetUpMenus()
{
	short	i;
		
	myMenus[AppleMenu] = GetMenu(AppleID);
	AddResMenu(myMenus[AppleMenu],'DRVR');
	myMenus[FileMenu] = GetMenu(FileID);
	myMenus[EditMenu] = GetMenu(EditID);
	for (i = AppleMenu; i <= EditMenu; i++)
		InsertMenu(myMenus[i], 0);
	DrawMenuBar();
}			

/********************************************************************************/
/* 								ADJUST MENUS									*/
/*------------------------------------------------------------------------------*/
/* Ajustement de la barre de menus suivant la fentre de premier plan, juste	*/
/* lors d'un click dans la barre des menus										*/
/* 																				*/
/* Les paramtres de l'appel :													*/
/* ---------------------------													*/
/* 																				*/
/*		aucun																	*/
/* 																				*/
/********************************************************************************/

void AdjustMenus() 
{
	if ( IsAppWindow( FrontWindow() ) ) 
	{
		DisableItem( myMenus[EditMenu], UndoI );
		DisableItem( myMenus[EditMenu], CutI );
		DisableItem( myMenus[EditMenu], CopyI );
		DisableItem( myMenus[EditMenu], PasteI );
		DisableItem( myMenus[EditMenu], ClearI );
	} 
	else if ( IsDAWindow( FrontWindow() ) ) 
	{
		EnableItem( myMenus[EditMenu], UndoI );
		EnableItem( myMenus[EditMenu], CutI );
		EnableItem( myMenus[EditMenu], CopyI );
		EnableItem( myMenus[EditMenu], PasteI );
		EnableItem( myMenus[EditMenu], ClearI );
	}
}			

/********************************************************************************
* 								Show About										*
*------------------------------------------------------------------------------	*
* Affiche la fentre de About.													*
* 																				*
* Les paramtres de l'appel :													*
* ---------------------------													*
* 																				*
*		aucun																	*
* 																				*
********************************************************************************/
void ShowAbout()
{											
	DialogPtr	myDialog;					
	short		Hit; 							

	myDialog = GetNewDialog(AboutID,nil,(WindowPtr) -1);
	if (myDialog) {
		ModalDialog(nil,&Hit);
		DisposDialog(myDialog);
	} else {
		SysBeep(0);
	}
}
			
/********************************************************************************
* 								DO COMMAND										*
*------------------------------------------------------------------------------	*
* Execution des commandes du menu. Ici on se contente de grer les accessoires	*
* le About et la commande Quit.													*
* 																				*
* Les paramtres de l'appel :													*
* ---------------------------													*
* 																				*
*		mResult 	:	le menu et l'item slectionns							*
* 																				*
********************************************************************************/
void DoCommand(long mResult)
{
	short	theItem;
	short	theMenu;
	Str255	name;

	theItem = LoWord(mResult);
	theMenu = HiWord(mResult);
	
	switch ( theMenu )
	{
		case AppleID:
			if (theItem != 1) 
			{
				GetItem(myMenus[AppleMenu], theItem, name);
				theItem = OpenDeskAcc(name);
			} else {
				ShowAbout();
			}
			break;
		case EditID:
			SystemEdit( theItem-1 );
			break;
		case FileID: 
			doneFlag = true;
			break;
	}
	HiliteMenu(0);  
}

/********************************************************************************/
/* 								DO MOUSE DOWN									*/
/*------------------------------------------------------------------------------*/
/* Gre les clicks souris														*/
/* 																				*/
/* Les paramtres de l'appel :													*/
/* ---------------------------													*/
/* 																				*/
/*		aucun																	*/
/* 																				*/
/********************************************************************************/

void DoMouseDown() 
{
	WindowPtr whichWindow;
	
	switch ( FindWindow( myEvent.where, &whichWindow ) )
	{ 
		case inMenuBar:	
				AdjustMenus(); 
				DoCommand(MenuSelect(myEvent.where));
				break;
		case inSysWindow: 
				SystemClick(&myEvent, whichWindow);
				break;
		case inGoAway: 	 
				doneFlag = TrackGoAway(whichWindow, myEvent.where);
				break;
		case inDrag: 	 
				DragWindow(whichWindow, myEvent.where, &dragRect);
				break;
		case inContent: 	 
				if (whichWindow != FrontWindow())
					SelectWindow(whichWindow);
				else if (whichWindow == myWindow)
					TrackDialog(myWindow, myEvent.where);
				break;
	}
}


/********************************************************************************/
/* 								ADJUST CURSOR									*/
/*------------------------------------------------------------------------------*/
/* Ajuste le curseur suivant rgion et fentre									*/
/* 																				*/
/********************************************************************************/

void AdjustCursor() 
{
	if (foreGround && IsAppWindow(FrontWindow())) 
		SetCursor(&qd.arrow);
}


/********************************************************************************/
/* 								INITIALIZE										*/
/*------------------------------------------------------------------------------*/
/* Initialisations gnrales (hasWNE, foreGround, managers, fentre, Midi)		*/
/* 																				*/
/* Les paramtres de l'appel :													*/
/* ---------------------------													*/
/* 																				*/
/*		aucun																	*/
/* 																				*/
/********************************************************************************/

void Initialize()
{
	OSErr	err;

	err = SysEnvirons(1, &gMac);
	if (gMac.machineType < 0) AlertUser(3);
	hasWNE = TrapAvailable(_WaitNextEvent, ToolTrap);
	foreGround = true;
		
	InitGraf(&qd.thePort); 			   				/* initialise QuickDraw					*/
	InitFonts();					   				/* initialise le Font Manager			*/
	InitWindows();					   				/* initialise le Window Manager			*/
	InitDialogs(nil);				   				/* initialise le Dialog Manager			*/
	InitMenus();				   					/* initialise le Menu Manager			*/
	TEInit();									   	/* initialise le Text Edit Manager		*/
	InitCursor(); 					   				/* initialise le curseur de la souris	*/

	FlushEvents(everyEvent, 0);						/* efface vnements Mac en attente		*/

	SetUpMenus();									/* mise en place menus					*/
	SetUpMidi();									/* ouverture MidiShare					*/
	SetUpWindows();									/* initialisations fentre et listes	*/
}

/********************************************************************************/
/* 							Corps Principal										*/
/*------------------------------------------------------------------------------*/
/* Initialisation des managers, ouverture fentre, initialisations diverses	et	*/
/* tutti quanti, boucle principale.												*/
/* 																				*/
/********************************************************************************/

void main()
{
	Boolean b;
	
	MaxApplZone();
	Initialize();
	
	doneFlag = false;											
	while (!doneFlag) {											/* Main Loop				*/
		if (hasWNE) 
			b = WaitNextEvent(everyEvent, &myEvent, 0, nil);	/* no sleep, no mouseRgn	*/
		else {
			SystemTask();
	 		b = GetNextEvent(everyEvent, &myEvent);
		}
		AdjustCursor();											
		switch( myEvent.what ) {
			case nullEvent:
					DoIdle();
					break;
			case osEvent:
					switch (myEvent.message >> 24) {
						case suspendResumeMessage:
							foreGround = myEvent.message & resumeMask;
							break;
						case mouseMovedMessage:
							DoIdle();
							break;
					}
					break;
			case keyDown:
			case autoKey:
					theChar = myEvent.message & charCodeMask;
					if (myEvent.modifiers & cmdKey ) {
						AdjustMenus();
						DoCommand(MenuKey(theChar));
					}
					break;
			case mouseDown:
					DoMouseDown();
					break;
			case updateEvt:
					if ( IsAppWindow( (WindowPtr)myEvent.message ) ) {
						BeginUpdate( (WindowPtr)myEvent.message );
						if ( !EmptyRgn( ((WindowPtr)myEvent.message)->visRgn ) ) {
							SetPort( (WindowPtr)myEvent.message );
							DrawDialog( (WindowPtr)myEvent.message );
						}
						EndUpdate( (WindowPtr)myEvent.message );
					}
					break;
		}
	}
	MidiClose(myRefNum);						
	CloseAllWinds( FrontWindow() );
}
